/* BeaSTM Add-On (c) Thorbjörn Jemander */

#pragma export on
extern "C" 
{
bool Process( uint16 *argSourceData, uint16 *argDestinationData, uint32 XSize, uint32 YSize );
}
#pragma export reset

float	CalcColumnSumPowerData(	uint32 argColumn,
								uint16 *argData,
								uint32 argXSize,
								uint32 argYSize,
								uint32 argPower );
void	CopySubtractedColumn(	uint32 argRow,
								uint16 *argDestData,
								uint16 *argSourceData,
								uint32 argXSize,
								uint32 argYSize,
								float m,
								float k );
float	CalcSumPowerData(		uint32 argRowNumber,
								uint16 *argData,
								uint32 argXSize,
								uint32 argPower );
void	CopySubtractedLine(		uint32 argRow,
								uint16 *argDestData,
								uint16 *argSourceData,
								uint32 argXSize,
								float m,
								float k );

bool
Process( uint16 *argSourceData, uint16 *argDestinationData, uint32 argXSize, uint32 argYSize )
{
	float	m,
			k,
			N = argYSize,
			n = N - 1,
			E = n + 1,
			D = n*(n+1)/2,
			C = n*n*n/3 + n*n/2 + n/6,
			H,
			G;

	uint32	row;
			
	for( row = 0; row < argYSize; row++ )
	{
		H = CalcSumPowerData( row, argSourceData, argXSize, 0 );
		G = CalcSumPowerData( row, argSourceData, argXSize, 1 );

		m = (H*C-G*D)/(E*C-D*D);
		k = (G-D*m)/C;
		
		CopySubtractedLine( row, argSourceData, argSourceData, argXSize, m, k );
	}
	uint32	column;
			
	for( column = 0; column < argXSize; column++ )
	{
		H = CalcColumnSumPowerData( column, argSourceData, argXSize, argYSize, 0 );
		G = CalcColumnSumPowerData( column, argSourceData, argXSize, argYSize, 1 );

		m = (H*C-G*D)/(E*C-D*D);
		k = (G-D*m)/C;
		
		CopySubtractedColumn( column, argDestinationData, argSourceData, argXSize, argYSize, m, k );
	}

	return true;
}

float
CalcSumPowerData( uint32 argRowNumber, uint16 *argData, uint32 argXSize, uint32 argPower )
{
	float 	i;
	float	Sum = 0;
	
	switch( argPower )
	{
		case 0:
			for( i=0; i<argXSize; i++ )
				Sum += argData[ argRowNumber * argXSize + (uint32)i ];
		break;
		case 1:
			for( i=0; i<argXSize; i++ )
				Sum += i*argData[ argRowNumber * argXSize + (uint32)i ];
		break;
		case 2:
			for( i=0; i<argXSize; i++ )
				Sum += i*i*argData[ argRowNumber * argXSize + (uint32)i ];
		break;
		default:
			for( i=0; i<argXSize; i++ )
				Sum += pow(i,argPower)*argData[ argRowNumber * argXSize + (uint32)i ];
	}
	
	return Sum;
}

void
CopySubtractedLine( uint32 argRow, uint16 *argDestData, uint16 *argSourceData, uint32 argXSize, float m, float k )
{
	uint32 	i;
	
	for( i=0; i<argXSize; i++ )
		argDestData[ argRow * argXSize + i ] = 
			argSourceData[ argRow * argXSize + i ] - ( i*k );
}

float
CalcColumnSumPowerData(	uint32 argColumn,
						uint16 *argData,
						uint32 argXSize,
						uint32 argYSize,
						uint32 argPower )
{
	float 	i;
	float	Sum = 0;
	
	switch( argPower )
	{
		case 0:
			for( i=0; i<argYSize; i++ )
				Sum += argData[ argColumn + argXSize * (uint32)i ];
		break;
		case 1:
			for( i=0; i<argYSize; i++ )
				Sum += i*argData[ argColumn + argXSize * (uint32)i ];
		break;
		case 2:
			for( i=0; i<argYSize; i++ )
				Sum += i*i*argData[ argColumn + argXSize * (uint32)i ];
		break;
		default:
			for( i=0; i<argYSize; i++ )
				Sum += pow(i,argPower)*argData[ argColumn * argXSize + (uint32)i ];
	}
	
	return Sum;
}

void
CopySubtractedColumn(	uint32 argColumn,
						uint16 *argDestData,
						uint16 *argSourceData,
						uint32 argXSize,
						uint32 argYSize,
						float m,
						float k )
{
	uint32 	i;
	
	for( i=0; i<argYSize; i++ )
		argDestData[ argColumn + argXSize * i ] = 
			argSourceData[ argColumn + argXSize * i ] - ( i*k );
}
